_boseos_info=0;
static variable Counts = Assoc_Type[Int_Type, 0];
static variable Times = Assoc_Type[Double_Type, 0.0];
static variable Stack_Depth = 0;
static variable Times_Stack = Double_Type[4096];
static variable Lines_Stack = String_Type[4096];
static variable Counts_Stack = Int_Type[4096];
%_debug_info = 1;

static variable Num_Counts;
static variable Overhead_Per_Call = 3e-05;


static define compute_time ()
{
   variable tms = times ();
   return tms.tms_utime + tms.tms_stime;
}

static define bos_handler (file, line)
{
   file = sprintf ("%s:%d", file, line);
   Counts[file] += 1;
   variable s = Stack_Depth;
   Stack_Depth++;
   Lines_Stack[s] = file;
   Counts_Stack[s] = Num_Counts;
   Num_Counts++;
   Times_Stack[s] = compute_time ();
}

static define eos_handler ()
{
   Stack_Depth--;
   variable file = Lines_Stack[Stack_Depth];
   Times[file] += compute_time () - Times_Stack[Stack_Depth]
     - Overhead_Per_Call * (Num_Counts - Counts_Stack[Stack_Depth]);
}

public define enable_profiling ()
{
   ()=_set_bos_handler (&bos_handler);
   ()=_set_eos_handler (&eos_handler);
   Stack_Depth = 0;
   Times = Assoc_Type[Double_Type, 0.0];
   Counts = Assoc_Type[Int_Type, 0];
   Counts_Stack = Int_Type[4096];
   Num_Counts = 0;
   _boseos_info = 3;
}

public define disable_profiling ()
{
   ()=_set_bos_handler (NULL);
   ()=_set_eos_handler (NULL);
   _boseos_info = 0;
}

static define calibrate_fun_0 (n)
{
   loop (n) 
     {
	() = 1;
	() = 1;
	() = 1;
	() = 1;
     }
}
_boseos_info = 3;
static define calibrate_fun_1 (n)
{
   loop (n) 
     {
	() = 1;
	() = 1;
	() = 1;
	() = 1;
     }
}
_boseos_info = 0;

public define profile_calibrate (n1)
{
   variable t0, t1;
   variable n0 = 1000*n1;
   
   Overhead_Per_Call = 0.0;
   tic();
   calibrate_fun_0 (n0);
   t0 = toc;
   
   enable_profiling ();
   calibrate_fun_1 (n1);
   disable_profiling ();
   t1 = sum(assoc_get_values (Times));
   variable factor = sum (assoc_get_values (Counts))/n1;
   % For large n, the bos/eos hooks were called 4n times.  Assume the following:
   %
   %   t1/n1 = t0/n0 + 4*overhead_per_call.
   Overhead_Per_Call = (t1/n1-t0/n0)/factor;
   return Overhead_Per_Call;
}

public define write_profile_info (fp)
{
   if (typeof (fp) == String_Type)
     fp = fopen (fp, "w");

   variable keys = assoc_get_keys (Times);
   variable values = assoc_get_values (Times);
   variable i = array_sort (values);
   variable key;
   foreach key (keys[i])
     {
	variable c = Counts[key];
	variable t = Times[key];
	() = fprintf (fp, "%s:\t%d\t%g\n", key, c, t);
     }
}
	
provide ("profile");
